@@ -2,7 +2,7 @@
                 routines, but not by user code
    Copyright (c) 2001-2008 Philip Kendall, Darren Salt
 
-   $Id: internals.h 3775 2008-09-28 03:28:22Z fredm $
+   $Id: internals.h 4122 2010-02-21 05:45:00Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,6 +43,20 @@
 #define GCC_PRINTF( fmtstring, args )
 #endif				/* #ifdef __GNUC__ */
 
+#ifdef _MSC_VER
+#if _MSC_VER > 1200		/* VC2005 or later */
+#define __func__ __FUNCTION__
+#else				/* #if _MSC_VER > 1200 */
+#define __func__ "__func__"
+#endif				/* _MSC_VER > 1200 */
+#endif				/* #ifdef _MSC_VER */
+
+/* VC6 lacks M_LN2, and VS2003+ require _USE_MATH_DEFINES defined before math.h
+ */
+#ifndef M_LN2
+#define M_LN2 0.693147180559945309417
+#endif
+
 /* On Win32 systems, map snprintf -> _snprintf, strcasecmp -> _stricmp and
    strncasecmp -> _strnicmp */
 #if !defined(HAVE_SNPRINTF) && defined(HAVE__SNPRINTF)
@@ -80,13 +94,6 @@ libspectrum_uncompress_file( unsigned char **new_buffer, size_t *new_length,
 			     const unsigned char *old_buffer,
 			     size_t old_length, const char *old_filename );
 
-libspectrum_error
-libspectrum_zlib_inflate( const libspectrum_byte *gzptr, size_t gzlength,
-			  libspectrum_byte **outptr, size_t *outlength );
-libspectrum_error
-libspectrum_zlib_compress( const libspectrum_byte *data, size_t length,
-			   libspectrum_byte **gzptr, size_t *gzlength );
-
 libspectrum_error
 libspectrum_gzip_inflate( const libspectrum_byte *gzptr, size_t gzlength,
 			  libspectrum_byte **outptr, size_t *outlength );
