@@ -1,7 +1,7 @@
 /* libspectrum.c: Some general routines
-   Copyright (c) 2001-2004 Philip Kendall, Darren Salt, Fredrick Meunier
+   Copyright (c) 2001-2009 Philip Kendall, Darren Salt, Fredrick Meunier
 
-   $Id: libspectrum.c 3943 2009-01-10 14:24:56Z pak21 $
+   $Id: libspectrum.c 4148 2010-08-25 21:20:50Z pak21 $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -223,6 +223,7 @@ libspectrum_machine_name( libspectrum_machine type )
   switch( type ) {
   case LIBSPECTRUM_MACHINE_16:       return "Spectrum 16K";
   case LIBSPECTRUM_MACHINE_48:       return "Spectrum 48K";
+  case LIBSPECTRUM_MACHINE_48_NTSC:  return "Spectrum 48K (NTSC)";
   case LIBSPECTRUM_MACHINE_TC2048:   return "Timex TC2048";
   case LIBSPECTRUM_MACHINE_TC2068:   return "Timex TC2068";
   case LIBSPECTRUM_MACHINE_TS2068:   return "Timex TS2068";
@@ -347,7 +348,7 @@ libspectrum_machine_capabilities( libspectrum_machine type )
     break;
   }
 
-  /* TRDOS-style disk */
+  /* Built-in TRDOS-style disk */
   switch( type ) {
   case LIBSPECTRUM_MACHINE_PENT:
   case LIBSPECTRUM_MACHINE_PENT512: case LIBSPECTRUM_MACHINE_PENT1024:
@@ -409,6 +410,7 @@ libspectrum_machine_capabilities( libspectrum_machine type )
 
   /* NTSC display */
   switch( type ) {
+  case LIBSPECTRUM_MACHINE_48_NTSC:
   case LIBSPECTRUM_MACHINE_TS2068:
     capabilities |= LIBSPECTRUM_MACHINE_CAPABILITY_NTSC; break;
   default:
@@ -547,6 +549,9 @@ libspectrum_identify_file_raw( libspectrum_id_t *type, const char *filename,
       { LIBSPECTRUM_ID_DISK_TD0,      "td0", 3, "TD",               0, 2, 4 },
       { LIBSPECTRUM_ID_DISK_TD0,      "td0", 3, "td",               0, 2, 4 },
 
+      { LIBSPECTRUM_ID_DISK_OPD,      "opd", 3, NULL,		    0, 0, 0 },
+      { LIBSPECTRUM_ID_DISK_OPD,      "opu", 3, NULL,		    0, 0, 0 },
+
       { -1, NULL, 0, NULL, 0, 0, 0 }, /* End marker */
 
     };
@@ -669,6 +674,9 @@ libspectrum_identify_class( libspectrum_class_t *libspectrum_class,
   case LIBSPECTRUM_ID_DISK_MGT:
     *libspectrum_class = LIBSPECTRUM_CLASS_DISK_PLUSD; return 0;
 
+  case LIBSPECTRUM_ID_DISK_OPD:
+    *libspectrum_class = LIBSPECTRUM_CLASS_DISK_OPUS; return 0;
+
   case LIBSPECTRUM_ID_DISK_SCL:
   case LIBSPECTRUM_ID_DISK_TRD:
     *libspectrum_class = LIBSPECTRUM_CLASS_DISK_TRDOS; return 0;
