@@ -1,7 +1,7 @@
 /* make-perl.c: Generate a perl script to create the libspectrum_* typedefs
    Copyright (c) 2002-2003 Philip Kendall, Darren Salt
 
-   $Id: make-perl.c 3716 2008-07-18 03:25:13Z specu $
+   $Id: make-perl.c 4123 2010-02-22 11:56:10Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -89,12 +89,18 @@ int main(void)
   if( sizeof( long ) == 8 ) {
     printf( "typedef unsigned long libspectrum_qword;\n" );
     printf( "typedef   signed long libspectrum_signed_qword;\n" );
+#if defined(_MSC_VER) && _MSC_VER <= 1200
+  } else {
+    printf( "typedef unsigned __int64 libspectrum_qword;\n" );
+    printf( "typedef   signed __int64 libspectrum_signed_qword;\n" );
+#else
   } else if( sizeof( long long ) == 8 ) {
     printf( "typedef unsigned long long libspectrum_qword;\n" );
     printf( "typedef   signed long long libspectrum_signed_qword;\n" );
   } else {
     fprintf( stderr, "No plausible 64 bit types found\n" );
     return 1;
+#endif
   }
 
 #endif				/* #ifdef HAVE_STDINT_H */
@@ -144,42 +150,46 @@ int main(void)
 "						 gconstpointer	b);\n"
 "\n"
 "\n"
-"GSList WIN32_DLL *g_slist_insert_sorted	(GSList		*list,\n"
+"WIN32_DLL GSList *g_slist_insert_sorted	(GSList		*list,\n"
 "						 gpointer	 data,\n"
 "						 GCompareFunc	 func);\n"
 "\n"
-"GSList WIN32_DLL *g_slist_insert		(GSList		*list,\n"
+"WIN32_DLL GSList *g_slist_insert		(GSList		*list,\n"
 "			 			 gpointer	 data,\n"
 "			 			 gint		 position);\n"
 "\n"
-"GSList WIN32_DLL *g_slist_append		(GSList		*list,\n"
+"WIN32_DLL GSList *g_slist_append		(GSList		*list,\n"
 "						 gpointer	 data);\n"
 "\n"
-"GSList WIN32_DLL *g_slist_prepend		(GSList		*list,\n"
+"WIN32_DLL GSList *g_slist_prepend		(GSList		*list,\n"
 "						 gpointer	 data);\n"
 "\n"
-"GSList WIN32_DLL *g_slist_remove		(GSList		*list,\n"
+"WIN32_DLL GSList *g_slist_remove		(GSList		*list,\n"
 "						 gpointer	 data);\n"
 "\n"
-"GSList WIN32_DLL *g_slist_delete_link		(GSList		*list,\n"
+"WIN32_DLL GSList *g_slist_last			(GSList		*list);\n"
+"\n"
+"WIN32_DLL GSList *g_slist_reverse		(GSList		*list);\n"
+"\n"
+"WIN32_DLL GSList *g_slist_delete_link		(GSList		*list,\n"
 "				 		 GSList		*link);\n"
 "\n"
-"guint	WIN32_DLL g_slist_length		(GSList *list);\n"
+"WIN32_DLL guint	g_slist_length		(GSList *list);\n"
 "\n"
-"void	WIN32_DLL g_slist_foreach		(GSList		*list,\n"
+"WIN32_DLL void	g_slist_foreach			(GSList		*list,\n"
 "						 GFunc		 func,\n"
 "						 gpointer	 user_data);\n"
 "\n"
-"void	WIN32_DLL g_slist_free			(GSList		*list);\n"
+"WIN32_DLL void	g_slist_free			(GSList		*list);\n"
 "\n"
-"GSList WIN32_DLL *g_slist_nth		(GSList		*list,\n"
+"WIN32_DLL GSList *g_slist_nth		(GSList		*list,\n"
 "					 guint		n);\n"
 "\n"
-"GSList WIN32_DLL * g_slist_find_custom	(GSList		*list,\n"
+"WIN32_DLL GSList *g_slist_find_custom	(GSList		*list,\n"
 "					 gpointer	data,\n"
 "					 GCompareFunc	func );\n"
 "\n"
-"gint	WIN32_DLL g_slist_position	(GSList		*list,\n"
+"WIN32_DLL gint	g_slist_position	(GSList		*list,\n"
 "					 GSList		*llink);\n"
 "\n"
 "typedef struct _GHashTable	GHashTable;\n"
@@ -190,31 +200,31 @@ int main(void)
 "						 gpointer	value,\n"
 "						 gpointer	user_data);\n"
 "\n"
-"gint	WIN32_DLL g_int_equal (gconstpointer   v,\n"
+"WIN32_DLL gint	g_int_equal (gconstpointer   v,\n"
 "			       gconstpointer   v2);\n"
-"guint	WIN32_DLL g_int_hash  (gconstpointer   v);\n"
+"WIN32_DLL guint	g_int_hash  (gconstpointer   v);\n"
 "\n"
-"gint	WIN32_DLL g_str_equal (gconstpointer   v,\n"
+"WIN32_DLL gint	g_str_equal (gconstpointer   v,\n"
 "			       gconstpointer   v2);\n"
-"guint	WIN32_DLL g_str_hash  (gconstpointer   v);\n"
+"WIN32_DLL guint	g_str_hash  (gconstpointer   v);\n"
 "\n"
-"GHashTable WIN32_DLL *g_hash_table_new	(GHashFunc	 hash_func,\n"
+"WIN32_DLL GHashTable *g_hash_table_new	(GHashFunc	 hash_func,\n"
 "					 GCompareFunc	 key_compare_func);\n"
 "\n"
-"void	WIN32_DLL g_hash_table_destroy	(GHashTable	*hash_table);\n"
+"WIN32_DLL void	g_hash_table_destroy	(GHashTable	*hash_table);\n"
 "\n"
-"void	WIN32_DLL g_hash_table_insert	(GHashTable	*hash_table,\n"
+"WIN32_DLL void	g_hash_table_insert	(GHashTable	*hash_table,\n"
 "					 gpointer	 key,\n"
 "					 gpointer	 value);\n"
 "\n"
-"gpointer WIN32_DLL g_hash_table_lookup	(GHashTable	*hash_table,\n"
+"WIN32_DLL gpointer g_hash_table_lookup	(GHashTable	*hash_table,\n"
 "					 gconstpointer	 key);\n"
 "\n"
-"guint	WIN32_DLL g_hash_table_foreach_remove	(GHashTable	*hash_table,\n"
+"WIN32_DLL guint	g_hash_table_foreach_remove	(GHashTable	*hash_table,\n"
 "						 GHRFunc	 func,\n"
 "						 gpointer	 user_data);\n"
 "\n"
-"guint	WIN32_DLL g_hash_table_size (GHashTable	*hash_table);\n"
+"WIN32_DLL guint	g_hash_table_size (GHashTable	*hash_table);\n"
 "\n"
 "typedef struct _GArray GArray;\n"
 "\n"
@@ -228,12 +238,12 @@ int main(void)
 "  size_t allocated;\n"
 "};\n"
 "\n"
-"GArray* WIN32_DLL g_array_new( gboolean zero_terminated, gboolean clear,\n"
+"WIN32_DLL GArray* g_array_new( gboolean zero_terminated, gboolean clear,\n"
 "		      guint element_size );\n"
 "#define g_array_append_val(a,v) g_array_append_vals( a, &(v), 1 );\n"
-"GArray* WIN32_DLL g_array_append_vals( GArray *array, gconstpointer data, guint len );\n"
+"WIN32_DLL GArray* g_array_append_vals( GArray *array, gconstpointer data, guint len );\n"
 "#define g_array_index(a,t,i) (*(((t*)a->data)+i))\n"
-"GArray* WIN32_DLL g_array_set_size( GArray *array, guint length );\n"
+"WIN32_DLL GArray* g_array_set_size( GArray *array, guint length );\n"
 "\n" );
   if( sizeof( void* ) == sizeof( int ) ) {
     printf( "#define GINT_TO_POINTER(i)      ((gpointer)  (i))\n" );
@@ -288,7 +298,14 @@ int main(void)
 #ifdef HAVE_ZLIB_H
 
   printf( "\n/* we support snapshots etc. requiring zlib (e.g. compressed szx) */\n" );
-  printf( "#define	LIBSPECTRUM_SUPPORTS_ZLIB_COMPRESSION	(1)\n" );
+  printf( "#define	LIBSPECTRUM_SUPPORTS_ZLIB_COMPRESSION	(1)\n\n" );
+  printf( "/* zlib (de)compression routines */\n\n" );
+  printf( "WIN32_DLL libspectrum_error\n" );
+  printf( "libspectrum_zlib_inflate( const libspectrum_byte *gzptr, size_t gzlength,\n" );
+  printf( "			  libspectrum_byte **outptr, size_t *outlength );\n\n" );
+  printf( "WIN32_DLL libspectrum_error\n" );
+  printf( "libspectrum_zlib_compress( const libspectrum_byte *data, size_t length,\n" );
+  printf( "			   libspectrum_byte **gzptr, size_t *gzlength );\n\n" );
 
 #endif				/* #ifdef HAVE_ZLIB_H */
 
