@@ -10,7 +10,7 @@
 
    Modified by Philip Kendall 2004-2008.
 
-   $Id: ghash.c 3705 2008-06-30 21:28:15Z fredm $
+   $Id: ghash.c 4119 2010-02-21 04:40:36Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -254,7 +254,7 @@ g_str_hash (gconstpointer v)
 {
   /* 31 bit hash function */
   const signed char *p = v;
-  uint32_t h = *p;
+  libspectrum_dword h = *p;
 
   if (h)
     for (p += 1; *p != '\0'; p++)
