@@ -1,7 +1,7 @@
 /* gslist.c: Minimal replacement for GSList
    Copyright (c) 2001-2004 Matan Ziv-Av, Philip Kendall, Marek Januszewski
 
-   $Id: gslist.c 3698 2008-06-30 15:12:02Z pak21 $
+   $Id: gslist.c 4039 2009-06-27 06:04:44Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -246,6 +246,18 @@ GSList* g_slist_delete_link	(GSList		*list,
   return list;
 }
 
+GSList*
+g_slist_last (GSList *list)
+{
+  if (list)
+    {
+      while (list->next)
+      list = list->next;
+    }
+
+  return list;
+}
+
 guint
 g_slist_length (GSList *list)
 {
@@ -285,6 +297,24 @@ void	g_slist_free		(GSList		*list) {
     }
 }
 
+GSList*
+g_slist_reverse (GSList *list)
+{
+  GSList *prev = NULL;
+
+  while (list)
+    {
+      GSList *next = list->next;
+
+      list->next = prev;
+
+      prev = list;
+      list = next;
+    }
+
+  return prev;
+}
+
 GSList* g_slist_nth		(GSList		*list,
 				 guint		n) {
   for( ; n; n-- ) {
