@@ -1,7 +1,7 @@
 /* rzx.c: routines for dealing with .rzx files
    Copyright (c) 2002-2008 Philip Kendall
 
-   $Id: rzx.c 3785 2008-10-22 18:26:20Z pak21 $
+   $Id: rzx.c 4157 2010-09-12 02:11:22Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -671,13 +671,12 @@ libspectrum_rzx_read( libspectrum_rzx *rzx, const libspectrum_byte *buffer,
 {
   libspectrum_error error;
   const libspectrum_byte *ptr, *end;
-  int uncompressed;
   libspectrum_byte *new_buffer;
   libspectrum_id_t raw_type;
   libspectrum_class_t class;
 
   /* Find out if this file needs decompression */
-  uncompressed = 0; new_buffer = NULL;
+  new_buffer = NULL;
 
   error = libspectrum_identify_file_raw( &raw_type, NULL, buffer, length );
   if( error ) return error;
@@ -691,8 +690,8 @@ libspectrum_rzx_read( libspectrum_rzx *rzx, const libspectrum_byte *buffer,
 
     error = libspectrum_uncompress_file( &new_buffer, &new_length, NULL,
 					 raw_type, buffer, length, NULL );
+    if( error ) return error;
     buffer = new_buffer; length = new_length;
-    uncompressed = 1;
   }
 
   ptr = buffer; end = buffer + length;
@@ -861,8 +860,9 @@ rzx_read_snapshot( libspectrum_rzx *rzx, const libspectrum_byte **ptr,
   /* We don't handle 'links' to external snapshots. I really think these
      are just more trouble than they're worth */
   if( flags & 0x01 ) {
-    libspectrum_print_error( LIBSPECTRUM_ERROR_UNKNOWN,
-			     "rzx_read_snapshot: skipping external snapshot" );
+    /* TODO log error if this isn't the first block as that can't be easily
+       covered by prompting the user for a snapshot file (but should be
+       unlikely to encounter in reality?) */
     (*ptr) += blocklength - 9;
     return LIBSPECTRUM_ERROR_NONE;
   }
