@@ -1,7 +1,7 @@
 /* snapshot.c: Snapshot handling routines
-   Copyright (c) 2001-2008 Philip Kendall, Darren Salt
+   Copyright (c) 2001-2009 Philip Kendall, Darren Salt
 
-   $Id: snapshot.c 3701 2008-06-30 20:32:56Z pak21 $
+   $Id: snapshot.c 4077 2009-09-01 13:05:45Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -109,6 +109,7 @@ libspectrum_snap_alloc( void )
 
   libspectrum_snap_set_beta_active( snap, 0 );
   libspectrum_snap_set_beta_paged( snap, 0 );
+  libspectrum_snap_set_beta_drive_count( snap, 0 );
   libspectrum_snap_set_beta_custom_rom( snap, 0 );
   libspectrum_snap_set_beta_direction( snap, 0 );
   libspectrum_snap_set_beta_system( snap, 0 );
@@ -120,6 +121,7 @@ libspectrum_snap_alloc( void )
 
   libspectrum_snap_set_plusd_active( snap, 0 );
   libspectrum_snap_set_plusd_paged( snap, 0 );
+  libspectrum_snap_set_plusd_drive_count( snap, 0 );
   libspectrum_snap_set_plusd_custom_rom( snap, 0 );
   libspectrum_snap_set_plusd_direction( snap, 0 );
   libspectrum_snap_set_plusd_control( snap, 0 );
@@ -130,6 +132,24 @@ libspectrum_snap_alloc( void )
   libspectrum_snap_set_plusd_rom( snap, 0, NULL );
   libspectrum_snap_set_plusd_ram( snap, 0, NULL );
 
+  libspectrum_snap_set_opus_active( snap, 0 );
+  libspectrum_snap_set_opus_paged ( snap, 0 );
+  libspectrum_snap_set_opus_drive_count( snap, 0 );
+  libspectrum_snap_set_opus_custom_rom( snap, 0 );
+  libspectrum_snap_set_opus_direction ( snap, 0 );
+  libspectrum_snap_set_opus_track ( snap, 0 );
+  libspectrum_snap_set_opus_sector( snap, 0 );
+  libspectrum_snap_set_opus_data  ( snap, 0 );
+  libspectrum_snap_set_opus_status( snap, 0 );
+  libspectrum_snap_set_opus_data_reg_a( snap, 0 );
+  libspectrum_snap_set_opus_data_dir_a( snap, 0 );
+  libspectrum_snap_set_opus_control_a ( snap, 0 );
+  libspectrum_snap_set_opus_data_reg_b( snap, 0 );
+  libspectrum_snap_set_opus_data_dir_b( snap, 0 );
+  libspectrum_snap_set_opus_control_b ( snap, 0 );
+  libspectrum_snap_set_opus_rom( snap, 0, NULL );
+  libspectrum_snap_set_opus_ram( snap, 0, NULL );
+
   libspectrum_snap_set_zxatasp_active( snap, 0 );
   libspectrum_snap_set_zxatasp_upload( snap, 0 );
   libspectrum_snap_set_zxatasp_writeprotect( snap, 0 );
@@ -182,6 +202,10 @@ libspectrum_snap_alloc( void )
     libspectrum_snap_set_divide_ram( snap, i, NULL );
   }
 
+  libspectrum_snap_set_fuller_box_active( snap, 0 );
+
+  libspectrum_snap_set_melodik_active( snap, 0 );
+
   return snap;
 }
 
@@ -238,7 +262,6 @@ libspectrum_snap_read( libspectrum_snap *snap, const libspectrum_byte *buffer,
   libspectrum_class_t class;
   libspectrum_byte *new_buffer;
   libspectrum_error error;
-  int uncompressed;
 
   /* If we don't know what sort of file this is, make a best guess */
   if( type == LIBSPECTRUM_ID_UNKNOWN ) {
@@ -265,7 +288,7 @@ libspectrum_snap_read( libspectrum_snap *snap, const libspectrum_byte *buffer,
   }
 
   /* Find out if this file needs decompression */
-  uncompressed = 0; new_buffer = NULL;
+  new_buffer = NULL;
 
   error = libspectrum_identify_file_raw( &raw_type, filename, buffer, length );
   if( error ) return error;
@@ -279,8 +302,8 @@ libspectrum_snap_read( libspectrum_snap *snap, const libspectrum_byte *buffer,
 
     error = libspectrum_uncompress_file( &new_buffer, &new_length, NULL,
 					 raw_type, buffer, length, NULL );
+    if( error ) return error;
     buffer = new_buffer; length = new_length;
-    uncompressed = 1;
   }
 
   switch( type ) {
