@@ -1,7 +1,7 @@
 /* tap.c: Routines for handling .tap files
    Copyright (c) 2001-2003 Philip Kendall
 
-   $Id: tap.c 3370 2007-11-30 07:48:32Z zubzero $
+   $Id: tap.c 3784 2008-10-22 12:36:07Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -55,7 +55,6 @@ internal_tap_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
 		   const size_t length, libspectrum_id_t type )
 {
   libspectrum_tape_block *block;
-  libspectrum_error error;
   size_t data_length, buf_length; libspectrum_byte *data;
 
   const libspectrum_byte *ptr, *end;
@@ -75,9 +74,7 @@ internal_tap_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
       return LIBSPECTRUM_ERROR_CORRUPT;
     }
 
-    /* Get memory for a new block */
-    error = libspectrum_tape_block_alloc( &block, LIBSPECTRUM_TAPE_BLOCK_ROM );
-    if( error ) return error;
+    block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_ROM );
 
     /* Get the length, and move along the buffer */
     data_length = ptr[0] + ptr[1] * 0x100;
@@ -96,7 +93,7 @@ internal_tap_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
     /* Have we got enough bytes left in buffer? */
     if( end - ptr < (ptrdiff_t)buf_length ) {
       libspectrum_tape_clear( tape );
-      free( block );
+      libspectrum_free( block );
       libspectrum_print_error(
         LIBSPECTRUM_ERROR_CORRUPT,
         "libspectrum_tap_read: not enough data in buffer"
@@ -105,14 +102,7 @@ internal_tap_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
     }
 
     /* Allocate memory for the data */
-    data = malloc( data_length * sizeof( libspectrum_byte ) );
-    if( !data ) {
-      libspectrum_tape_clear( tape );
-      free( block );
-      libspectrum_print_error( LIBSPECTRUM_ERROR_MEMORY,
-			       "libspectrum_tap_read: out of memory" );
-      return LIBSPECTRUM_ERROR_MEMORY;
-    }
+    data = libspectrum_malloc( data_length * sizeof( *data ) );
     libspectrum_tape_block_set_data( block, data );
 
     /* Copy the block data across */
@@ -138,22 +128,12 @@ internal_tap_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
     /* Give a 1s pause after each block */
     libspectrum_tape_block_set_pause( block, 1000 );
 
-    /* Finally, put the block into the block list */
-    error = libspectrum_tape_append_block( tape, block );
-    if( error ) { libspectrum_tape_block_free( block ); return error; }
-
+    libspectrum_tape_append_block( tape, block );
   }
 
   return LIBSPECTRUM_ERROR_NONE;
 }
 
-libspectrum_error
-libspectrum_tap_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
-		      const size_t length )
-{
-  return internal_tap_read( tape, buffer, length, LIBSPECTRUM_ID_TAPE_TAP );
-}
-
 libspectrum_error
 internal_tap_write( libspectrum_byte **buffer, size_t *length,
 		    libspectrum_tape *tape, libspectrum_id_t type )
@@ -174,19 +154,19 @@ internal_tap_write( libspectrum_byte **buffer, size_t *length,
 
     case LIBSPECTRUM_TAPE_BLOCK_ROM:
       error = write_rom( block, buffer, &ptr, length, type );
-      if( error != LIBSPECTRUM_ERROR_NONE ) { free( *buffer ); return error; }
+      if( error != LIBSPECTRUM_ERROR_NONE ) { libspectrum_free( *buffer ); return error; }
       done = 1;
       break;
 
     case LIBSPECTRUM_TAPE_BLOCK_TURBO:
       error = write_turbo( block, buffer, &ptr, length, type );
-      if( error != LIBSPECTRUM_ERROR_NONE ) { free( *buffer ); return error; }
+      if( error != LIBSPECTRUM_ERROR_NONE ) { libspectrum_free( *buffer ); return error; }
       done = 1;
       break;
 
     case LIBSPECTRUM_TAPE_BLOCK_PURE_DATA:
       error = write_pure_data( block, buffer, &ptr, length, type );
-      if( error != LIBSPECTRUM_ERROR_NONE ) { free( *buffer ); return error; }
+      if( error != LIBSPECTRUM_ERROR_NONE ) { libspectrum_free( *buffer ); return error; }
       done = 1;
       break;
 
@@ -198,7 +178,7 @@ internal_tap_write( libspectrum_byte **buffer, size_t *length,
     case LIBSPECTRUM_TAPE_BLOCK_LOOP_END:
     case LIBSPECTRUM_TAPE_BLOCK_RLE_PULSE:
       error = skip_block( block, "conversion almost certainly won't work" );
-      if( error != LIBSPECTRUM_ERROR_NONE ) { free( *buffer ); return 1; }
+      if( error != LIBSPECTRUM_ERROR_NONE ) { libspectrum_free( *buffer ); return 1; }
       done = 1;
       break;
 
@@ -206,7 +186,7 @@ internal_tap_write( libspectrum_byte **buffer, size_t *length,
     case LIBSPECTRUM_TAPE_BLOCK_JUMP:
     case LIBSPECTRUM_TAPE_BLOCK_SELECT:
       error = skip_block( block, "conversion may not work" );
-      if( error != LIBSPECTRUM_ERROR_NONE ) { free( *buffer ); return 1; }
+      if( error != LIBSPECTRUM_ERROR_NONE ) { libspectrum_free( *buffer ); return 1; }
       done = 1;
       break;
 
@@ -220,13 +200,13 @@ internal_tap_write( libspectrum_byte **buffer, size_t *length,
     case LIBSPECTRUM_TAPE_BLOCK_CUSTOM:
     case LIBSPECTRUM_TAPE_BLOCK_CONCAT:
       error = skip_block( block, NULL );
-      if( error != LIBSPECTRUM_ERROR_NONE ) { free( *buffer ); return 1; }
+      if( error != LIBSPECTRUM_ERROR_NONE ) { libspectrum_free( *buffer ); return 1; }
       done = 1;
       break;
     }
 
     if( !done ) {
-      if( *length ) free( *buffer );
+      if( *length ) libspectrum_free( *buffer );
       libspectrum_print_error(
         LIBSPECTRUM_ERROR_LOGIC,
         "libspectrum_tap_write: unknown block type 0x%02x",
@@ -243,13 +223,6 @@ internal_tap_write( libspectrum_byte **buffer, size_t *length,
   return LIBSPECTRUM_ERROR_NONE;
 }
 
-libspectrum_error
-libspectrum_tap_write( libspectrum_byte **buffer, size_t *length,
-		       libspectrum_tape *tape )
-{
-  return internal_tap_write( buffer, length, tape, LIBSPECTRUM_ID_TAPE_TAP );
-}
-
 static libspectrum_error
 write_rom( libspectrum_tape_block *block, libspectrum_byte **buffer,
 	   libspectrum_byte **ptr, size_t *length, libspectrum_id_t type )
@@ -310,7 +283,6 @@ write_tap_block( libspectrum_byte **buffer, libspectrum_byte **ptr,
 		 size_t *length, libspectrum_byte *data, size_t data_length,
 		 libspectrum_id_t type )
 {
-  libspectrum_error error;
   size_t buf_length;
 
   /* Discard the parity byte for STA files */
@@ -330,8 +302,7 @@ write_tap_block( libspectrum_byte **buffer, libspectrum_byte **ptr,
     data_length -= 2;
   }
 
-  error = libspectrum_make_room( buffer, 2 + buf_length, ptr, length );
-  if( error != LIBSPECTRUM_ERROR_NONE ) return error;
+  libspectrum_make_room( buffer, 2 + buf_length, ptr, length );
 
   /* Write out the length and the data */
   *(*ptr)++ =   data_length & 0x00ff;
