@@ -1,7 +1,7 @@
 /* tape.c: Routines for handling tape files
    Copyright (c) 2001-2008 Philip Kendall, Darren Salt, Fredrick Meunier
 
-   $Id: tape.c 3775 2008-09-28 03:28:22Z fredm $
+   $Id: tape.c 4077 2009-09-01 13:05:45Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,6 +37,9 @@ struct libspectrum_tape {
   /* All the blocks */
   GSList* blocks;
 
+  /* The last block */
+  GSList* last_block;
+
   /* The state of the current block */
   libspectrum_tape_block_state state;
 
@@ -112,6 +115,7 @@ libspectrum_tape_alloc( void )
 {
   libspectrum_tape *tape = libspectrum_malloc( sizeof( *tape ) );
   tape->blocks = NULL;
+  tape->last_block = NULL;
   libspectrum_tape_iterator_init( &(tape->state.current_block), tape );
   tape->state.loop_block = NULL;
   return tape;
@@ -160,7 +164,6 @@ libspectrum_tape_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
   libspectrum_class_t class;
   libspectrum_byte *new_buffer;
   libspectrum_error error;
-  int uncompressed;
 
   /* If we don't know what sort of file this is, make a best guess */
   if( type == LIBSPECTRUM_ID_UNKNOWN ) {
@@ -178,7 +181,7 @@ libspectrum_tape_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
   }
 
   /* Find out if this file needs decompression */
-  uncompressed = 0; new_buffer = NULL;
+  new_buffer = NULL;
 
   error = libspectrum_identify_file_raw( &raw_type, filename, buffer, length );
   if( error ) return error;
@@ -192,8 +195,8 @@ libspectrum_tape_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
 
     error = libspectrum_uncompress_file( &new_buffer, &new_length, NULL,
 					 raw_type, buffer, length, NULL );
+    if( error ) return error;
     buffer = new_buffer; length = new_length;
-    uncompressed = 1;
   }
 
   switch( type ) {
@@ -297,6 +300,7 @@ const int LIBSPECTRUM_TAPE_FLAGS_LEVEL_LOW  = 1 << 4; /* Set level low */
 const int LIBSPECTRUM_TAPE_FLAGS_LEVEL_HIGH = 1 << 5; /* Set level high */
 const int LIBSPECTRUM_TAPE_FLAGS_LENGTH_SHORT = 1 << 6;
 const int LIBSPECTRUM_TAPE_FLAGS_LENGTH_LONG = 1 << 7;
+const int LIBSPECTRUM_TAPE_FLAGS_TAPE       = 1 << 8; /* End of tape */
 
 libspectrum_error
 libspectrum_tape_get_next_edge_internal( libspectrum_dword *tstates,
@@ -442,6 +446,7 @@ libspectrum_tape_get_next_edge_internal( libspectrum_dword *tstates,
 	 then `rewind' to the start) */
       if( libspectrum_tape_iterator_current( it->current_block ) == NULL ) {
 	*flags |= LIBSPECTRUM_TAPE_FLAGS_STOP;
+        *flags |= LIBSPECTRUM_TAPE_FLAGS_TAPE;
         libspectrum_tape_iterator_init( &(it->current_block), tape );
       }
     }
@@ -833,8 +838,8 @@ libspectrum_tape_raw_data_next_bit( libspectrum_tape_raw_data_block *block,
       if( state->bytes_through_block == block->length )
 	break;
     }
-  } while( (block->data[state->bytes_through_block] << state->bits_through_byte
-            & 0x80 ) != state->last_bit) ;
+  } while( ( block->data[state->bytes_through_block] <<
+             state->bits_through_byte & 0x80 ) != state->last_bit) ;
 
   state->bit_tstates = length * block->bit_length;
   state->last_bit ^= 0x80;
@@ -1037,6 +1042,13 @@ libspectrum_tape_peek_next_block( libspectrum_tape *tape )
   return block ? block : tape->blocks->data;
 }
 
+/* Peek at the last block on the tape */
+libspectrum_tape_block WIN32_DLL *
+libspectrum_tape_peek_last_block( libspectrum_tape *tape )
+{
+  return tape->last_block ? tape->last_block->data : NULL;
+}
+
 /* Cause the next block on the tape to be active, initialise it
    and return it */
 libspectrum_tape_block*
@@ -1103,7 +1115,13 @@ void
 libspectrum_tape_append_block( libspectrum_tape *tape,
 			       libspectrum_tape_block *block )
 {
-  tape->blocks = g_slist_append( tape->blocks, (gpointer)block );
+  if( tape->blocks == NULL ) {
+    tape->blocks = g_slist_append( tape->blocks, (gpointer)block );
+    tape->last_block = tape->blocks;
+  } else {
+    tape->last_block =
+      g_slist_append( tape->last_block, (gpointer)block )->next;
+  }
 
   /* If we previously didn't have a tape loaded ( implied by
      tape->current_block == NULL ), set up so that we point to the
@@ -1120,6 +1138,7 @@ libspectrum_tape_remove_block( libspectrum_tape *tape,
 {
   if( it->data ) libspectrum_tape_block_free( it->data );
   tape->blocks = g_slist_delete_link( tape->blocks, it );
+  tape->last_block = g_slist_last( tape->blocks );
 }
 
 libspectrum_error
@@ -1128,6 +1147,7 @@ libspectrum_tape_insert_block( libspectrum_tape *tape,
 			       size_t position )
 {
   tape->blocks = g_slist_insert( tape->blocks, block, position );
+  tape->last_block = g_slist_last( tape->blocks );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
