@@ -1,7 +1,7 @@
 /* tzx_read.c: Routines for reading .tzx files
    Copyright (c) 2001-2007 Philip Kendall, Darren Salt
 
-   $Id: tzx_read.c 3071 2007-07-20 17:12:49Z pak21 $
+   $Id: tzx_read.c 3784 2008-10-22 12:36:07Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -95,7 +95,7 @@ tzx_read_custom( libspectrum_tape *tape, const libspectrum_byte **ptr,
 static libspectrum_error
 tzx_read_concat( const libspectrum_byte **ptr, const libspectrum_byte *end );
 
-static libspectrum_error
+static void
 tzx_read_empty_block( libspectrum_tape *tape, libspectrum_tape_type id );
 
 static libspectrum_error
@@ -187,8 +187,7 @@ internal_tzx_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
       if( error ) { libspectrum_tape_clear( tape ); return error; }
       break;
     case LIBSPECTRUM_TAPE_BLOCK_GROUP_END:
-      error = tzx_read_empty_block( tape, id );
-      if( error ) { libspectrum_tape_clear( tape ); return error; }
+      tzx_read_empty_block( tape, id );
       break;
     case LIBSPECTRUM_TAPE_BLOCK_JUMP:
       error = tzx_read_jump( tape, &ptr, end );
@@ -199,8 +198,7 @@ internal_tzx_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
       if( error ) { libspectrum_tape_clear( tape ); return error; }
       break;
     case LIBSPECTRUM_TAPE_BLOCK_LOOP_END:
-      error = tzx_read_empty_block( tape, id );
-      if( error ) { libspectrum_tape_clear( tape ); return error; }
+      tzx_read_empty_block( tape, id );
       break;
 
     case LIBSPECTRUM_TAPE_BLOCK_SELECT:
@@ -253,13 +251,6 @@ internal_tzx_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
   return LIBSPECTRUM_ERROR_NONE;
 }
 
-libspectrum_error
-libspectrum_tzx_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
-		      const size_t length )
-{
-  return internal_tzx_read( tape, buffer, length );
-}
-
 static libspectrum_error
 tzx_read_rom_block( libspectrum_tape *tape, const libspectrum_byte **ptr,
 		    const libspectrum_byte *end )
@@ -276,9 +267,7 @@ tzx_read_rom_block( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block, LIBSPECTRUM_TAPE_BLOCK_ROM );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_ROM );
 
   /* Get the pause length */
   libspectrum_tape_block_set_pause( block, (*ptr)[0] + (*ptr)[1] * 0x100 );
@@ -286,17 +275,13 @@ tzx_read_rom_block( libspectrum_tape *tape, const libspectrum_byte **ptr,
 
   /* And the data */
   error = tzx_read_data( ptr, end, &length, 2, &data );
-  if( error ) { free( block ); return error; }
+  if( error ) { libspectrum_free( block ); return error; }
   libspectrum_tape_block_set_data_length( block, length );
   libspectrum_tape_block_set_data( block, data );
 
-  /* Finally, put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
-  /* And return with no error */
   return LIBSPECTRUM_ERROR_NONE;
-
 }
 
 static libspectrum_error
@@ -316,9 +301,7 @@ tzx_read_turbo_block( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block, LIBSPECTRUM_TAPE_BLOCK_TURBO );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_TURBO );
 
   /* Get the metadata */
   libspectrum_tape_block_set_pilot_length( block,
@@ -346,17 +329,13 @@ tzx_read_turbo_block( libspectrum_tape *tape, const libspectrum_byte **ptr,
 
   /* Read the data in */
   error = tzx_read_data( ptr, end, &length, 3, &data );
-  if( error ) { free( block ); return error; }
+  if( error ) { libspectrum_free( block ); return error; }
   libspectrum_tape_block_set_data_length( block, length );
   libspectrum_tape_block_set_data( block, data );
 
-  /* Finally, put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
-  /* And return with no error */
   return LIBSPECTRUM_ERROR_NONE;
-
 }
 
 static libspectrum_error
@@ -364,7 +343,6 @@ tzx_read_pure_tone( libspectrum_tape *tape, const libspectrum_byte **ptr,
 		    const libspectrum_byte *end )
 {
   libspectrum_tape_block* block;
-  libspectrum_error error;
 
   /* Check we've got enough bytes */
   if( end - (*ptr) < 4 ) {
@@ -373,10 +351,7 @@ tzx_read_pure_tone( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block,
-					LIBSPECTRUM_TAPE_BLOCK_PURE_TONE );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_PURE_TONE );
 
   /* Read in the data, and move along */
   libspectrum_tape_block_set_pulse_length( block,
@@ -385,11 +360,8 @@ tzx_read_pure_tone( libspectrum_tape *tape, const libspectrum_byte **ptr,
   libspectrum_tape_block_set_count( block, (*ptr)[0] + (*ptr)[1] * 0x100 );
   (*ptr) += 2;
   
-  /* Finally, put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
-  /* And return with no error */
   return LIBSPECTRUM_ERROR_NONE;
 }
 
@@ -399,7 +371,7 @@ tzx_read_pulses_block( libspectrum_tape *tape, const libspectrum_byte **ptr,
 {
   libspectrum_tape_block* block;
   libspectrum_dword *lengths;
-  libspectrum_error error; size_t i, count;
+  size_t i, count;
 
   /* Check the count byte exists */
   if( (*ptr) == end ) {
@@ -410,10 +382,7 @@ tzx_read_pulses_block( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block,
-					LIBSPECTRUM_TAPE_BLOCK_PULSES );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_PULSES );
 
   /* Get the count byte */
   count = **ptr; (*ptr)++;
@@ -421,7 +390,7 @@ tzx_read_pulses_block( libspectrum_tape *tape, const libspectrum_byte **ptr,
 
   /* Check enough data exists for every pulse */
   if( end - (*ptr) < (ptrdiff_t)( 2 * count ) ) {
-    free( block );
+    libspectrum_free( block );
     libspectrum_print_error(
       LIBSPECTRUM_ERROR_CORRUPT,
       "tzx_read_pulses_block: not enough data in buffer"
@@ -429,14 +398,7 @@ tzx_read_pulses_block( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Allocate memory for the actual list of lengths */
-  lengths = malloc( count * sizeof( libspectrum_dword ) );
-  if( lengths == NULL ) {
-    free( block );
-    libspectrum_print_error( LIBSPECTRUM_ERROR_MEMORY,
-			     "tzx_read_pulses_block: out of memory" );
-    return LIBSPECTRUM_ERROR_MEMORY;
-  }
+  lengths = libspectrum_malloc( count * sizeof( *lengths ) );
 
   /* Copy the data across */
   for( i = 0; i < count; i++ ) {
@@ -444,11 +406,8 @@ tzx_read_pulses_block( libspectrum_tape *tape, const libspectrum_byte **ptr,
   }
   libspectrum_tape_block_set_pulse_lengths( block, lengths );
 
-  /* Put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
-  /* And return with no error */
   return LIBSPECTRUM_ERROR_NONE;
 }
 
@@ -470,10 +429,7 @@ tzx_read_pure_data( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block,
-					LIBSPECTRUM_TAPE_BLOCK_PURE_DATA );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_PURE_DATA );
 
   /* Get the metadata */
   libspectrum_tape_block_set_bit0_length( block,
@@ -488,17 +444,13 @@ tzx_read_pure_data( libspectrum_tape *tape, const libspectrum_byte **ptr,
 
   /* And the actual data */
   error = tzx_read_data( ptr, end, &length, 3, &data );
-  if( error ) { free( block ); return error; }
+  if( error ) { libspectrum_free( block ); return error; }
   libspectrum_tape_block_set_data_length( block, length );
   libspectrum_tape_block_set_data( block, data );
 
-  /* Finally, put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
-  /* And return with no error */
   return LIBSPECTRUM_ERROR_NONE;
-
 }
 
 static libspectrum_error
@@ -516,10 +468,7 @@ tzx_read_raw_data (libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block,
-					LIBSPECTRUM_TAPE_BLOCK_RAW_DATA );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_RAW_DATA );
 
   /* Get the metadata */
   libspectrum_tape_block_set_bit_length( block,
@@ -531,13 +480,11 @@ tzx_read_raw_data (libspectrum_tape *tape, const libspectrum_byte **ptr,
 
   /* And the actual data */
   error = tzx_read_data( ptr, end, &length, 3, &data );
-  if( error ) { free( block ); return error; }
+  if( error ) { libspectrum_free( block ); return error; }
   libspectrum_tape_block_set_data_length( block, length );
   libspectrum_tape_block_set_data( block, data );
 
-  /* Finally, put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   /* And return with no error */
   return LIBSPECTRUM_ERROR_NONE;
@@ -585,10 +532,7 @@ tzx_read_generalised_data( libspectrum_tape *tape,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  error =
-    libspectrum_tape_block_alloc( &block,
-				  LIBSPECTRUM_TAPE_BLOCK_GENERALISED_DATA );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_GENERALISED_DATA );
 
   libspectrum_tape_block_zero( block );
 
@@ -620,21 +564,8 @@ tzx_read_generalised_data( libspectrum_tape *tape,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  symbols = malloc( symbol_count * sizeof( *symbols ) );
-  if( !symbols ) {
-    libspectrum_tape_block_free( block );
-    libspectrum_print_error( LIBSPECTRUM_ERROR_MEMORY, "%s:%d", __func__,
-			     __LINE__ );
-    return LIBSPECTRUM_ERROR_MEMORY;
-  }
-
-  repeats = malloc( symbol_count * sizeof( *repeats ) );
-  if( !repeats ) {
-    libspectrum_tape_block_free( block );
-    libspectrum_print_error( LIBSPECTRUM_ERROR_MEMORY, "%s:%d", __func__,
-			     __LINE__ );
-    return LIBSPECTRUM_ERROR_MEMORY;
-  }
+  symbols = libspectrum_malloc( symbol_count * sizeof( *symbols ) );
+  repeats = libspectrum_malloc( symbol_count * sizeof( *repeats ) );
 
   for( i = 0; i < symbol_count; i++ ) {
     symbols[ i ] = **ptr; (*ptr)++;
@@ -664,16 +595,10 @@ tzx_read_generalised_data( libspectrum_tape *tape,
   data_count = ( ( bits_per_symbol * symbol_count ) + 7 ) / 8;
   data_size = data_count * sizeof( *data );
 
-  data = malloc( data_size );
-  if( !data ) {
-    libspectrum_tape_block_free( block );
-    libspectrum_print_error( LIBSPECTRUM_ERROR_MEMORY, "%s:%d", __func__,
-			     __LINE__ );
-    return LIBSPECTRUM_ERROR_MEMORY;
-  }
+  data = libspectrum_malloc( data_size );
 
   if( *ptr + data_size > end || *ptr + data_size < *ptr ) {
-    free( data );
+    libspectrum_free( data );
     libspectrum_tape_block_free( block );
     libspectrum_print_error( LIBSPECTRUM_ERROR_CORRUPT,
 			     "%s: data extends beyond end of block", __func__ );
@@ -693,8 +618,7 @@ tzx_read_generalised_data( libspectrum_tape *tape,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
@@ -704,7 +628,6 @@ tzx_read_pause( libspectrum_tape *tape, const libspectrum_byte **ptr,
 		const libspectrum_byte *end )
 {
   libspectrum_tape_block *block;
-  libspectrum_error error;
 
   /* Check the pause actually exists */
   if( end - (*ptr) < 2 ) {
@@ -713,18 +636,13 @@ tzx_read_pause( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block,
-					LIBSPECTRUM_TAPE_BLOCK_PAUSE );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_PAUSE );
 
   /* Get the pause length */
   libspectrum_tape_block_set_pause( block, (*ptr)[0] + (*ptr)[1] * 0x100 );
   (*ptr) += 2;
 
-  /* Put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   /* And return */
   return LIBSPECTRUM_ERROR_NONE;
@@ -747,19 +665,14 @@ tzx_read_group_start( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block,
-					LIBSPECTRUM_TAPE_BLOCK_GROUP_START );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_GROUP_START );
 
   /* Read in the description of the group */
   error = tzx_read_string( ptr, end, &name );
-  if( error ) { free( block ); return error; }
+  if( error ) { libspectrum_free( block ); return error; }
   libspectrum_tape_block_set_text( block, name );
 			  
-  /* Finally, put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
@@ -770,7 +683,6 @@ tzx_read_jump( libspectrum_tape *tape, const libspectrum_byte **ptr,
 {
   libspectrum_tape_block *block;
   int offset;
-  libspectrum_error error;
   
   /* Check the offset exists */
   if( end - (*ptr) < 2 ) {
@@ -779,18 +691,14 @@ tzx_read_jump( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block, LIBSPECTRUM_TAPE_BLOCK_JUMP );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_JUMP );
 
   /* Get the offset */
   offset = (*ptr)[0] + (*ptr)[1] * 0x100; (*ptr) += 2;
   if( offset >= 32768 ) offset -= 65536;
   libspectrum_tape_block_set_offset( block, offset);
 
-  /* Finally, put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
@@ -800,7 +708,6 @@ tzx_read_loop_start( libspectrum_tape *tape, const libspectrum_byte **ptr,
 		     const libspectrum_byte *end )
 {
   libspectrum_tape_block *block;
-  libspectrum_error error;
   
   /* Check the count exists */
   if( end - (*ptr) < 2 ) {
@@ -811,18 +718,13 @@ tzx_read_loop_start( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block,
-					LIBSPECTRUM_TAPE_BLOCK_LOOP_START );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_LOOP_START );
 
   /* Get the repeat count */
   libspectrum_tape_block_set_count( block, (*ptr)[0] + (*ptr)[1] * 0x100 );
   (*ptr) += 2;
 
-  /* Finally, put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
@@ -854,32 +756,17 @@ tzx_read_select( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block,
-					LIBSPECTRUM_TAPE_BLOCK_SELECT );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_SELECT );
 
   /* Get the number of selections */
   count = **ptr; (*ptr)++;
   libspectrum_tape_block_set_count( block, count );
 
   /* Allocate memory */
-  offsets = malloc( count * sizeof( int ) );
-  if( !offsets ) {
-    free( block );
-    libspectrum_print_error( LIBSPECTRUM_ERROR_MEMORY, 
-			     "tzx_read_select: out of memory" );
-    return LIBSPECTRUM_ERROR_MEMORY;
-  }
+  offsets = libspectrum_malloc( count * sizeof( *offsets ) );
   libspectrum_tape_block_set_offsets( block, offsets );
 
-  descriptions = malloc( count * sizeof( char* ) );
-  if( !descriptions ) {
-    free( offsets ); free( block );
-    libspectrum_print_error( LIBSPECTRUM_ERROR_MEMORY,
-			     "tzx_read_select: out of memory" );
-    return LIBSPECTRUM_ERROR_MEMORY;
-  }
+  descriptions = libspectrum_malloc( count * sizeof( *descriptions ) );
   libspectrum_tape_block_set_texts( block, descriptions );
 
   /* Read in the data */
@@ -887,8 +774,8 @@ tzx_read_select( libspectrum_tape *tape, const libspectrum_byte **ptr,
 
     /* Check we've got the offset and a length byte */
     if( end - (*ptr) < 3 ) {
-      for( j = 0; j < i; j++ ) free( descriptions[j] );
-      free( descriptions ); free( offsets ); free( block );
+      for( j = 0; j < i; j++ ) libspectrum_free( descriptions[j] );
+      libspectrum_free( descriptions ); libspectrum_free( offsets ); libspectrum_free( block );
       libspectrum_print_error( LIBSPECTRUM_ERROR_CORRUPT,
 			       "tzx_read_select: not enough data in buffer" );
       return LIBSPECTRUM_ERROR_CORRUPT;
@@ -900,16 +787,14 @@ tzx_read_select( libspectrum_tape *tape, const libspectrum_byte **ptr,
     /* Get the description of this selection */
     error = tzx_read_string( ptr, end, &descriptions[i] );
     if( error ) {
-      for( j = 0; j < i; j++ ) free( descriptions[j] );
-      free( descriptions ); free( offsets ); free( block );
+      for( j = 0; j < i; j++ ) libspectrum_free( descriptions[j] );
+      libspectrum_free( descriptions ); libspectrum_free( offsets ); libspectrum_free( block );
       return error;
     }
 
   }
 
-  /* Finally, put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
@@ -919,7 +804,6 @@ tzx_read_stop( libspectrum_tape *tape, const libspectrum_byte **ptr,
 	       const libspectrum_byte *end )
 {
   libspectrum_tape_block *block;
-  libspectrum_error error;
 
   /* Check the length field exists */
   if( end - (*ptr) < 4 ) {
@@ -931,14 +815,9 @@ tzx_read_stop( libspectrum_tape *tape, const libspectrum_byte **ptr,
   /* But then just skip over it, as I don't care what it is */
   (*ptr) += 4;
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block,
-					LIBSPECTRUM_TAPE_BLOCK_STOP48 );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_STOP48 );
 
-  /* Put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   return LIBSPECTRUM_ERROR_NONE;
 }  
@@ -958,19 +837,14 @@ tzx_read_comment( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block,
-					LIBSPECTRUM_TAPE_BLOCK_COMMENT );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_COMMENT );
 
   /* Get the actual comment */
   error = tzx_read_string( ptr, end, &text );
-  if( error ) { free( block ); return error; }
+  if( error ) { libspectrum_free( block ); return error; }
   libspectrum_tape_block_set_text( block, text );
 
-  /* Finally, put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
@@ -990,22 +864,17 @@ tzx_read_message( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block,
-					LIBSPECTRUM_TAPE_BLOCK_MESSAGE );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_MESSAGE );
 
   /* Get the time */
   libspectrum_tape_block_set_pause( block, **ptr ); (*ptr)++;
 
   /* Get the message itself */
   error = tzx_read_string( ptr, end, &text );
-  if( error ) { free( block ); return error; }
+  if( error ) { libspectrum_free( block ); return error; }
   libspectrum_tape_block_set_text( block, text );
 
-  /* Finally, put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
@@ -1031,10 +900,7 @@ tzx_read_archive_info( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block,
-					LIBSPECTRUM_TAPE_BLOCK_ARCHIVE_INFO );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_ARCHIVE_INFO );
 
   /* Skip the length, as I don't care about it */
   (*ptr) += 2;
@@ -1044,22 +910,10 @@ tzx_read_archive_info( libspectrum_tape *tape, const libspectrum_byte **ptr,
   libspectrum_tape_block_set_count( block, count );
 
   /* Allocate memory */
-  ids = malloc( count * sizeof( int ) );
-  if( !ids ) {
-    free( block );
-    libspectrum_print_error( LIBSPECTRUM_ERROR_MEMORY,
-			     "tzx_read_archive_info: out of memory" );
-    return LIBSPECTRUM_ERROR_MEMORY;
-  }
+  ids = libspectrum_malloc( count * sizeof( *ids ) );
   libspectrum_tape_block_set_ids( block, ids );
 
-  strings = malloc( count * sizeof( char* ) );
-  if( !strings ) {
-    free( ids ); free( block );
-    libspectrum_print_error( LIBSPECTRUM_ERROR_MEMORY,
-			     "tzx_read_archive_info: out of memory" );
-    return LIBSPECTRUM_ERROR_MEMORY;
-  }
+  strings = libspectrum_malloc( count * sizeof( *strings ) );
   libspectrum_tape_block_set_texts( block, strings );
 
   for( i = 0; i < count; i++ ) {
@@ -1067,8 +921,8 @@ tzx_read_archive_info( libspectrum_tape *tape, const libspectrum_byte **ptr,
     /* Must be ID byte and length byte */
     if( end - (*ptr) < 2 ) {
       size_t j;
-      for( j=0; j<i; j++ ) free( strings[j] );
-      free( strings ); free( ids ); free( block );
+      for( j=0; j<i; j++ ) libspectrum_free( strings[j] );
+      libspectrum_free( strings ); libspectrum_free( ids ); libspectrum_free( block );
       libspectrum_print_error(
         LIBSPECTRUM_ERROR_CORRUPT,
         "tzx_read_archive_info: not enough data in buffer"
@@ -1083,16 +937,14 @@ tzx_read_archive_info( libspectrum_tape *tape, const libspectrum_byte **ptr,
     error = tzx_read_string( ptr, end, &strings[i] );
     if( error ) {
       size_t j;
-      for( j = 0; j < i; j++ ) free( strings[j] );
-      free( strings ); free( ids ); free( block );
+      for( j = 0; j < i; j++ ) libspectrum_free( strings[j] );
+      libspectrum_free( strings ); libspectrum_free( ids ); libspectrum_free( block );
       return error;
     }
 
   }
 
-  /* Finally, put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
@@ -1104,7 +956,6 @@ tzx_read_hardware( libspectrum_tape *tape, const libspectrum_byte **ptr,
   libspectrum_tape_block* block;
   size_t i, count;
   int *types, *ids, *values;
-  libspectrum_error error;
 
   /* Check there's enough left in the buffer for the count byte */
   if( (*ptr) == end ) {
@@ -1113,10 +964,7 @@ tzx_read_hardware( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block,
-					LIBSPECTRUM_TAPE_BLOCK_HARDWARE );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_HARDWARE );
 
   /* Get the number of string */
   count = **ptr; (*ptr)++;
@@ -1124,38 +972,20 @@ tzx_read_hardware( libspectrum_tape *tape, const libspectrum_byte **ptr,
 
   /* Check there's enough data in the buffer for all the data */
   if( end - (*ptr) < 3 * (ptrdiff_t)count ) {
-    free( block );
+    libspectrum_free( block );
     libspectrum_print_error( LIBSPECTRUM_ERROR_CORRUPT,
 			     "tzx_read_hardware: not enough data in buffer" );
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
   /* Allocate memory */
-  types = malloc( count * sizeof( int ) );
-  if( !types ) {
-    free( block );
-    libspectrum_print_error( LIBSPECTRUM_ERROR_MEMORY,
-			     "tzx_read_hardware: out of memory" );
-    return LIBSPECTRUM_ERROR_MEMORY;
-  }
+  types = libspectrum_malloc( count * sizeof( *types ) );
   libspectrum_tape_block_set_types( block, types );
 
-  ids = malloc( count * sizeof( int ) );
-  if( !ids ) {
-    free( types ); free( block );
-    libspectrum_print_error( LIBSPECTRUM_ERROR_MEMORY,
-			     "tzx_read_hardware: out of memory" );
-    return LIBSPECTRUM_ERROR_MEMORY;
-  }
+  ids = libspectrum_malloc( count * sizeof( *ids ) );
   libspectrum_tape_block_set_ids( block, ids );
 
-  values = malloc( count * sizeof( int ) );
-  if( !values ) {
-    free( ids ); free( types ); free( block );
-    libspectrum_print_error( LIBSPECTRUM_ERROR_MEMORY,
-			     "tzx_read_hardware: out of memory" );
-    return LIBSPECTRUM_ERROR_MEMORY;
-  }
+  values = libspectrum_malloc( count * sizeof( *values ) );
   libspectrum_tape_block_set_values( block, values );
 
   /* Actually read in all the data */
@@ -1165,9 +995,7 @@ tzx_read_hardware( libspectrum_tape *tape, const libspectrum_byte **ptr,
     values[i] = **ptr; (*ptr)++;
   }
 
-  /* Finally, put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
@@ -1188,25 +1016,20 @@ tzx_read_custom( libspectrum_tape *tape, const libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_CORRUPT;
   }
 
-  /* Get a new block */
-  error = libspectrum_tape_block_alloc( &block,
-					LIBSPECTRUM_TAPE_BLOCK_CUSTOM );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_CUSTOM );
 
   /* Get the description */
-  description = calloc( 17, sizeof( libspectrum_byte ) );
+  description = libspectrum_malloc( 17 * sizeof( libspectrum_byte ) );
   memcpy( description, *ptr, 16 ); (*ptr) += 16; description[16] = '\0';
   libspectrum_tape_block_set_text( block, description );
 
   /* Read in the data */
   error = tzx_read_data( ptr, end, &length, 4, &data );
-  if( error ) { free( description ); free( block ); return error; }
+  if( error ) { libspectrum_free( description ); libspectrum_free( block ); return error; }
   libspectrum_tape_block_set_data_length( block, length );
   libspectrum_tape_block_set_data( block, data );
 
-  /* Finally, put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
+  libspectrum_tape_append_block( tape, block );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
@@ -1232,20 +1055,12 @@ tzx_read_concat( const libspectrum_byte **ptr, const libspectrum_byte *end )
   return LIBSPECTRUM_ERROR_NONE;
 }
   
-static libspectrum_error
+static void
 tzx_read_empty_block( libspectrum_tape *tape, libspectrum_tape_type id )
 {
   libspectrum_tape_block *block;
-  libspectrum_error error;
-
-  /* Get memory for a new block */
-  error = libspectrum_tape_block_alloc( &block, id ); if( error ) return error;
-
-  /* Put the block into the block list */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) { libspectrum_tape_block_free( block ); return error; }
-
-  return LIBSPECTRUM_ERROR_NONE;
+  block = libspectrum_tape_block_alloc( id );
+  libspectrum_tape_append_block( tape, block );
 }  
 
 static libspectrum_error
@@ -1276,14 +1091,7 @@ tzx_read_data( const libspectrum_byte **ptr, const libspectrum_byte *end,
   /* Allocate memory for the data; the check for *length is to avoid
      the implementation-defined of malloc( 0 ) */
   if( *length || padding ) {
-
-    *data = malloc( ( *length + padding ) * sizeof( libspectrum_byte ) );
-    if( !*data ) {
-      libspectrum_print_error( LIBSPECTRUM_ERROR_MEMORY,
-			       "tzx_read_data: out of memory" );
-      return LIBSPECTRUM_ERROR_MEMORY;
-    }
-
+    *data = libspectrum_malloc( ( *length + padding ) * sizeof( **data ) );
     /* Copy the block data across, and move along */
     memcpy( *data, *ptr, *length ); *ptr += *length;
   } else {
