@@ -2,7 +2,7 @@
    Copyright (c) 2001, 2002 Philip Kendall, Darren Salt
    Copyright (c) 2003 Fredrick Meunier
 
-   $Id: warajevo_read.c 3784 2008-10-22 12:36:07Z fredm $
+   $Id: warajevo_read.c 4197 2010-11-30 18:48:20Z pak21 $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -150,18 +150,17 @@ decompress_block( libspectrum_byte *dest, const libspectrum_byte *src,
 static libspectrum_dword
 lsb2dword( const libspectrum_byte *mem )
 {
-  return  mem[0]             +
-          mem[1] *     0x100 +
-          mem[2] *   0x10000 +
-          mem[3] * 0x1000000 ;
-
+  return ( mem[0] <<  0 ) |
+         ( mem[1] <<  8 ) |
+         ( mem[2] << 16 ) |
+         ( mem[3] << 24 );
 } 
   
 static libspectrum_word
 lsb2word( const libspectrum_byte *mem ) 
 {
-  return  mem[0]             +
-          mem[1] *     0x100 ;
+  return ( mem[0] << 0 ) |
+         ( mem[1] << 8 );
 }
 
 /* The main load function */
@@ -201,7 +200,7 @@ internal_warajevo_read( libspectrum_tape *tape,
     error = get_next_block( &offset, ptr, end, tape );
     if( error != LIBSPECTRUM_ERROR_NONE ) return error;
   }
-  
+
   return LIBSPECTRUM_ERROR_NONE;
 }
 
@@ -260,7 +259,8 @@ exec_command( libspectrum_byte *dest, const libspectrum_byte *src,
   for( i = 0; i < 8; i++ ) {
     bit = ( command_byte & ( 0x80 >> i ) ) ? 1 : 0;
 
-    error = add_bit_to_copy_command( dest, src, dest + to_write, bit, sp, bytes_written );
+    error = add_bit_to_copy_command( dest, src, dest + to_write, bit, sp,
+                                     bytes_written );
     if( error ) { return error; }
 
     if( *bytes_written >= to_write ) break;
@@ -511,6 +511,7 @@ read_raw_data( libspectrum_tape *tape, const libspectrum_byte *ptr,
 {
   libspectrum_tape_block *block =
     libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_RAW_DATA );
+  libspectrum_tape_block *last_block;
   libspectrum_error error;
   libspectrum_word compressed_size, decompressed_size;
   const libspectrum_byte *data = ptr + offset + 17;
@@ -541,7 +542,9 @@ read_raw_data( libspectrum_tape *tape, const libspectrum_byte *ptr,
 
     error = decompress_block( block_data, data, end,
 			      lsb2word( ptr + offset + 15 ), length );
-    if( error ) { libspectrum_free( block_data ); libspectrum_free( block ); return error; }
+    if( error ) {
+      libspectrum_free( block_data ); libspectrum_free( block ); return error;
+    }
   } else {
     /* Uncompressed block: just copy the data across */
     memcpy( block_data, data, length );
@@ -568,8 +571,44 @@ read_raw_data( libspectrum_tape *tape, const libspectrum_byte *ptr,
   libspectrum_tape_block_set_bits_in_last_byte( block,
 						status.bits.bits_used + 1 );
 
-  /* Put the block into the block list */
-  libspectrum_tape_append_block( tape, block );
+  /* Warajevo TAPs have a relatively short limit on RAW blocks of 64995 bytes
+     which is only 11.74 secs at 44100Hz.
+
+     As a result when these blocks are used they often are a sequence of these
+     short blocks as opposed to other formats which have a long single block
+     corresponding to the sampled section of tape.
+
+     This presents a problem for libspectrum which assumes that a pulse cannot
+     span two different blocks, so we will work around this limitation by
+     coalesing adjacent compatible RAW blocks to ensure the pulse structure is
+     preserved.
+     */
+
+  /* Check if the last block was also a raw block of the same sample rate
+     and with all 8 bits used in the last byte */
+  last_block = libspectrum_tape_peek_last_block( tape );
+  if( last_block &&
+      libspectrum_tape_block_type( last_block ) ==
+        LIBSPECTRUM_TAPE_BLOCK_RAW_DATA && 
+      libspectrum_tape_block_bit_length( last_block ) == bit_length &&
+      libspectrum_tape_block_bits_in_last_byte( last_block ) == 8 ) {
+    /* Combine the two blocks */
+    size_t new_length = libspectrum_tape_block_data_length( last_block ) + 
+                          length;
+    block_data = libspectrum_realloc( libspectrum_tape_block_data( last_block ),
+                                      new_length * sizeof( libspectrum_byte ) );
+
+    memcpy( block_data + libspectrum_tape_block_data_length( last_block ),
+            libspectrum_tape_block_data( block ), length );
+    libspectrum_tape_block_set_data( last_block, block_data );
+    libspectrum_tape_block_set_data_length( last_block, new_length );
+    libspectrum_tape_block_set_bits_in_last_byte( last_block,
+                                                  status.bits.bits_used + 1 );
+    libspectrum_tape_block_free( block );
+  } else {
+    /* Put the block into the block list */
+    libspectrum_tape_append_block( tape, block );
+  }
 
   /* And return with no error */
   return LIBSPECTRUM_ERROR_NONE;
