@@ -1,7 +1,7 @@
 /* wav.c: Routines for handling WAV raw audio files
    Copyright (c) 2007 Fredrick Meunier
 
-   $Id: wav.c 3708 2008-07-01 08:07:01Z pak21 $
+   $Id: wav.c 4100 2009-10-22 12:23:41Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -73,12 +73,22 @@ libspectrum_wav_read( libspectrum_tape *tape, const char *filename )
     return LIBSPECTRUM_ERROR_LOGIC;
   }
 
+  if( afSetVirtualChannels( handle, track, 1 ) ) {
+    afCloseFile( handle );
+    libspectrum_print_error(
+      LIBSPECTRUM_ERROR_LOGIC,
+      "libspectrum_wav_read: audiofile failed to set virtual channel count"
+    );
+    return LIBSPECTRUM_ERROR_LOGIC;
+  }
+
   length = afGetFrameCount( handle, track );
 
   tape_length = length;
   if( tape_length%8 ) tape_length += 8 - (tape_length%8);
 
-  buffer = libspectrum_calloc( tape_length, sizeof( *buffer ) );
+  buffer = libspectrum_calloc( tape_length * afGetChannels(handle, track),
+                               sizeof( *buffer ) );
 
   frames = afReadFrames( handle, track, buffer, length );
   if( frames == -1 ) {
