@@ -1,7 +1,7 @@
 /* z80.c: Routines for handling .z80 snapshots
-   Copyright (c) 2001-2004 Philip Kendall, Darren Salt, Fredrick Meunier
+   Copyright (c) 2001-2009 Philip Kendall, Darren Salt, Fredrick Meunier
 
-   $Id: z80.c 3784 2008-10-22 12:36:07Z fredm $
+   $Id: z80.c 4163 2010-09-29 21:29:00Z pak21 $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -110,6 +110,12 @@ static const libspectrum_word if2_left_f     = '5';
 static libspectrum_byte slt_signature[] = "\0\0\0SLT";
 static size_t slt_signature_length = 6;
 
+/* Bits used in extended header to flag that a Fuller Box is in use */
+static const libspectrum_byte fuller_box_flags = 0x44;
+
+/* Bits used in extended header to flag that a Melodik is in use */
+static const libspectrum_byte melodik_flags = 0x04;
+
 static libspectrum_error
 read_header( const libspectrum_byte *buffer, libspectrum_snap *snap,
 	     const libspectrum_byte **data, int *version, int *compressed );
@@ -329,7 +335,7 @@ read_header( const libspectrum_byte *buffer, libspectrum_snap *snap,
          b.. Any valid 128k identifier should be taken as +2
          c.. Any valid +3 identifier (7 or 8) should be taken as +2A
 
-       Support for Fuller Box / AY sound in 48k mode (not implemented)
+       Support for Fuller Box / AY sound in 48k mode
        
        Spectaculator recognises xzx's extension of setting bit 2 of byte 37 to
        specify AY sound in 48k mode. In addition, if this and also bit 6 is
@@ -353,6 +359,12 @@ read_header( const libspectrum_byte *buffer, libspectrum_snap *snap,
       }
     }
 
+    if( ( extra_header[5] & fuller_box_flags ) == fuller_box_flags ) {
+      libspectrum_snap_set_fuller_box_active( snap, 1 );
+    } else if( ( extra_header[5] & melodik_flags ) == melodik_flags ) {
+      libspectrum_snap_set_melodik_active( snap, 1 );
+    }
+
     capabilities =
       libspectrum_machine_capabilities( libspectrum_snap_machine( snap ) );
 
@@ -366,7 +378,9 @@ read_header( const libspectrum_byte *buffer, libspectrum_snap *snap,
       libspectrum_snap_set_out_scld_dec( snap, extra_header[4] );
     }
 
-    if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_AY ) {
+    if( libspectrum_snap_fuller_box_active( snap ) ||
+        libspectrum_snap_melodik_active( snap ) ||
+        capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_AY ) {
       libspectrum_snap_set_out_ay_registerport( snap, extra_header[ 6] );
       for( i = 0; i < 16; i++ ) {
 	libspectrum_snap_set_ay_registers( snap, i, extra_header[ 7 + i ] );
@@ -1125,6 +1139,10 @@ libspectrum_z80_write2( libspectrum_byte **buffer, size_t *length,
   if( libspectrum_snap_beta_active( snap ) )
     *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
 
+  /* .z80 format doesn't store Opus info at all */
+  if( libspectrum_snap_opus_active( snap ) )
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
+
   /* .z80 format doesn't store IDE interface info */
   if( libspectrum_snap_zxatasp_active( snap ) )
     *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
@@ -1250,28 +1268,28 @@ write_extended_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
 		       size_t *length, int *flags, libspectrum_snap *snap )
 {
   int i, bottom_16kb_ram;
+  libspectrum_byte hardware_flag = 0;      /* No special emulation features */
+  int second_memory_port = 0; /* By default, don't write the extra bytes */
+  libspectrum_byte machine_byte;
+  size_t header_length;
 
   libspectrum_dword quarter_states;
 
   int capabilities =
     libspectrum_machine_capabilities( libspectrum_snap_machine( snap ) );
 
-  /* +2 here to deal with the two length bytes */
-  libspectrum_make_room( buffer, LIBSPECTRUM_Z80_V3_LENGTH + 2, ptr, length );
-
-  libspectrum_write_word( ptr, LIBSPECTRUM_Z80_V3_LENGTH );
-  
-  libspectrum_write_word( ptr, libspectrum_snap_pc( snap ) );
-
   switch( libspectrum_snap_machine( snap ) ) {
+  case LIBSPECTRUM_MACHINE_48_NTSC:
+    *flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
+    /* fall through */
   case LIBSPECTRUM_MACHINE_16:
   case LIBSPECTRUM_MACHINE_48:
     if( libspectrum_snap_plusd_active( snap ) ) {
-      *(*ptr)++ = Z80_MACHINE_48_MGT;
+      machine_byte = Z80_MACHINE_48_MGT;
     } else if( libspectrum_snap_interface1_active( snap ) ) {
-      *(*ptr)++ = Z80_MACHINE_48_IF1;
+      machine_byte = Z80_MACHINE_48_IF1;
     } else {
-      *(*ptr)++ = Z80_MACHINE_48;
+      machine_byte = Z80_MACHINE_48;
     }
     break;
   case LIBSPECTRUM_MACHINE_SE:
@@ -1279,40 +1297,48 @@ write_extended_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
     /* fall through */
   case LIBSPECTRUM_MACHINE_128:
     if( libspectrum_snap_plusd_active( snap ) ) {
-      *(*ptr)++ = Z80_MACHINE_128_MGT;
+      machine_byte = Z80_MACHINE_128_MGT;
     } else if( libspectrum_snap_interface1_active( snap ) ) {
-      *(*ptr)++ = Z80_MACHINE_128_IF1;
+      machine_byte = Z80_MACHINE_128_IF1;
     } else {
-      *(*ptr)++ = Z80_MACHINE_128;
+      machine_byte = Z80_MACHINE_128;
     }
     break;
   case LIBSPECTRUM_MACHINE_PLUS3E:
     *flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MINOR_INFO_LOSS;
     /* fall through */
   case LIBSPECTRUM_MACHINE_PLUS3:
-    *(*ptr)++ = Z80_MACHINE_PLUS3; break;
+    second_memory_port = 1;
+    machine_byte = Z80_MACHINE_PLUS3;
+    break;
   case LIBSPECTRUM_MACHINE_PENT:
   case LIBSPECTRUM_MACHINE_PENT512:
   case LIBSPECTRUM_MACHINE_PENT1024:
-    *(*ptr)++ = Z80_MACHINE_PENTAGON; break;
+    second_memory_port = 1;
+    machine_byte = Z80_MACHINE_PENTAGON;
+    break;
   case LIBSPECTRUM_MACHINE_SCORP:
-    *(*ptr)++ = Z80_MACHINE_SCORPION; break;
+    second_memory_port = 1;
+    machine_byte = Z80_MACHINE_SCORPION;
+    break;
   case LIBSPECTRUM_MACHINE_PLUS2:
     if( libspectrum_snap_interface1_active( snap ) ) {
       *flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MINOR_INFO_LOSS;
     }
-    *(*ptr)++ = Z80_MACHINE_PLUS2; break;
+    machine_byte = Z80_MACHINE_PLUS2; break;
   case LIBSPECTRUM_MACHINE_PLUS2A:
-    *(*ptr)++ = Z80_MACHINE_PLUS2A; break;
+    second_memory_port = 1;
+    machine_byte = Z80_MACHINE_PLUS2A;
+    break;
   case LIBSPECTRUM_MACHINE_TC2048:
     if( libspectrum_snap_interface1_active( snap ) ) {
       *flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MINOR_INFO_LOSS;
     }
-    *(*ptr)++ = Z80_MACHINE_TC2048; break;
+    machine_byte = Z80_MACHINE_TC2048; break;
   case LIBSPECTRUM_MACHINE_TC2068:
-    *(*ptr)++ = Z80_MACHINE_TC2068; break;
+    machine_byte = Z80_MACHINE_TC2068; break;
   case LIBSPECTRUM_MACHINE_TS2068:
-    *(*ptr)++ = Z80_MACHINE_TS2068; break;
+    machine_byte = Z80_MACHINE_TS2068; break;
 
   case LIBSPECTRUM_MACHINE_UNKNOWN:
     libspectrum_print_error( LIBSPECTRUM_ERROR_UNKNOWN,
@@ -1321,6 +1347,17 @@ write_extended_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
     return LIBSPECTRUM_ERROR_UNKNOWN;
   }
 
+  header_length = second_memory_port ? LIBSPECTRUM_Z80_V3X_LENGTH
+                                     : LIBSPECTRUM_Z80_V3_LENGTH;
+
+  /* +2 here to deal with the two length bytes */
+  libspectrum_make_room( buffer, header_length + 2, ptr, length );
+
+  libspectrum_write_word( ptr, header_length );
+
+  libspectrum_write_word( ptr, libspectrum_snap_pc( snap ) );
+  *(*ptr)++ = machine_byte;
+
   if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_128_MEMORY ) {
     *(*ptr)++ = libspectrum_snap_out_128_memoryport( snap );
   } else if( capabilities & LIBSPECTRUM_MACHINE_CAPABILITY_TIMEX_MEMORY ) {
@@ -1344,11 +1381,23 @@ write_extended_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
   /* Support 16K snapshots via Spectaculator's extension; see the
      comment in read_header for details */
   if( libspectrum_snap_machine( snap ) == LIBSPECTRUM_MACHINE_16 ) {
-    *(*ptr)++ = 0x80;
-  } else {
-    *(*ptr)++ = '\0';		/* No special emulation features */
+    hardware_flag |= 0x80;
   }
 
+  /* Support Fuller Box in snapshots via Spectaculator's extension; see the
+     comment in read_header for details */
+  if( libspectrum_snap_fuller_box_active( snap ) ) {
+    /* Fuller is set by having bit 2 and 6 on */
+    hardware_flag |= fuller_box_flags;
+  }
+
+  if( libspectrum_snap_melodik_active( snap ) ) {
+    /* Melodik is set by having bit 2 on */
+    hardware_flag |= melodik_flags;
+  }
+
+  *(*ptr)++ = hardware_flag;
+
   *(*ptr)++ = libspectrum_snap_out_ay_registerport( snap );
   for( i = 0; i < 16; i++ )
     *(*ptr)++ = libspectrum_snap_ay_registers( snap, i );
@@ -1429,8 +1478,9 @@ write_extended_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
     for( i=52; i<55; i++ ) *(*ptr)++ = '\0';
   }
 
-  /* etc */
-  for( i=55; i<=LIBSPECTRUM_Z80_V3_LENGTH; i++ ) *(*ptr)++ = '\0';
+  if( second_memory_port ) {
+    *(*ptr)++ = libspectrum_snap_out_plus3_memoryport( snap );
+  }
 
   return LIBSPECTRUM_ERROR_NONE;
 }
